from json import loads

class WrongType(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)

class UnknownError(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)

class InvalidLink(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)

class IncorrectPassword(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)

class NotLoggined(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)

class AlreadyRegistered(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)

class BadMedia(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)

class NoWalletError(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)


class InvalidFile(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)


class InvalidEmail(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)


class EmailNotRegistered(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)



errors = {
	2009: EmailNotRegistered,
	2022: InvalidEmail,
	4604: InvalidLink,
	2010: IncorrectPassword,
	2038: AlreadyRegistered,
	7008: BadMedia,
	1000004: NoWalletError
}

def CheckException(data):
	try:
		data = loads(data)
		code = data["apiCode"]
	except:
		raise UnknownError(data)
	if code in errors: raise errors[code](data)
	else:raise UnknownError(data)


from hashlib import sha1, sha256 
from hmac import HMAC
from base64 import b64encode, urlsafe_b64encode
from typing import Optional
from os import urandom
from json import load, dumps
from uuid import uuid4

class Generator:
	def __init__(self):
		self.prefix = bytes.fromhex("01")
		self.SIG_KEY = bytes.fromhex("ebefcf164b887da7f924c948e1fc3e40faf230eb7d491c1de1150134b8517189")
		self.DEVICE_KEY = bytes.fromhex("dcfed9e64710da3a8458298424ff88e47375")

	def get_signable_header_keys(self) -> list[str]: return [
		"rawDeviceId", "rawDeviceIdTwo", "rawDeviceIdThree",
		"appType", "appVersion", "osType",
		"deviceType", "sId", "countryCode",
		"reqTime", "User-Agent", "contentRegion",
		"nonce", "carrierCountryCodes"
	]



	def _jwt_hs256(self, header: dict, data: dict, key: bytes) -> str:
		header_json = dumps(dict(alg="HS256", **header))
		data_json = dumps(data)
		body_str = f"{urlsafe_b64encode(header_json.encode('utf-8')).decode('utf-8')}" f".{urlsafe_b64encode(data_json.encode('utf-8')).decode('utf-8')}"

		sign = HMAC(key=key, msg=body_str.encode("utf-8"), digestmod=sha256).digest()
		return f"{body_str}.{urlsafe_b64encode(sign).decode('utf-8')}".replace("=", "")


	def signature(self, path: str, headers: dict, body) -> str:
		mac = HMAC(key=self.SIG_KEY, msg=path.encode("utf-8"), digestmod=sha256)
		for header in [headers[signable] for signable in self.get_signable_header_keys() if signable in headers]:
			mac.update(header.encode("utf-8"))
		if body: mac.update(body.encode("utf-8"))
		return b64encode(self.prefix + mac.digest()).decode("utf-8")

	def deviceId(self, installation_id: str = None):
		if not installation_id:installation_id=str(uuid4())
		prefix = self.prefix + sha1(installation_id.encode("utf-8")).digest()
		return ( prefix + sha1( prefix + sha1(self.DEVICE_KEY).digest() ).digest() ).hex()


	def generate_device_id_three(self, data: Optional[dict] = None) -> str:
		return "D" + self._jwt_hs256({
			"organization": "BU0gJ0gB5TFcCfN329Vx",
			"os": "android",
			"appId": "default",
			"encode": 1,
			"data": self._jwt_hs256(data or {
				"a1": "exception",
				"a6": "android",
				"a7": "3.0.6",
				"a2": "",
				"a10": "7.1.2",
				"a13": "ASUS_Z01QD",
				"a96": "BU0gJ0gB5TFcCfN329Vx",
				"a11": "default",
				"a98": b64encode("BU0gJ0gB5TFcCfN329Vx_bzcVDxF2PO"
								 "/ArnWpiOIWhT0WwjQ76FZ6BqAnhQpqI"
								 "OeGJYJvV5bcTZQ0lgjRQNAcyAqhRi7Ym"
								 "7tNesvah21ROA==".encode('utf-8')).decode("utf-8"),
				"e": "",
				"a97": "",
			}, {}, urandom(256)),
			"tn": "",
			"ep": "",
		}, {}, urandom(256))


from time import time
from uuid import uuid4

class Headers:
	def __init__(self, deviceId: str, sid: str = None, time_zone: int = 180, country_code: str = "en", language: str = "en-US"):
		self.sid = sid
		self.deviceId = deviceId
		self.time_zone = time_zone
		self.country_code = country_code
		self.language = language

	def Headers(self):

		headers = {
			"rawDeviceId": self.deviceId,
			"nonce": str(uuid4()),
			"Accept-Language": self.language,
			"countryCode": self.country_code.upper(),
			"carrierCountryCodes": self.country_code,
			"timeZone": str(self.time_zone),
			"reqTime": str(int(time() * 1000)),
			"sId": self.sid if self.sid else ''
		}

		return headers

	def get_persistent_headers(self) -> dict: return {
		"appType": "MainApp", "appVersion": "2.3.1",
		"osType": "1", "deviceType": "1", "flavor": "google",
		"User-Agent": "com.projz.z.android/1.24.7-13774 (Linux; U; Android 7.1.2; ASUS_Z01QD; Build/Asus-user 7.1.2 2017)"
	}



__all__ = ['Media', 'User', 'FromLink', 'ThreadList', 'Thread', 'Event', 'ActivateShop', 'WalletInfo']


class Media:
	def __init__(self, data: dict = {}):
		self.json = data
		self.mediaId = self.json.get('mediaId', None)
		self.baseUrl = self.json.get('baseUrl', None)
		self.resourceList = list()

		for res in self.json.get('resourceList', []):
			self.resourceList.append(self.Resources(res))


	class Resources:
		def __init__(self, data: dict = {}):
			self.json = data
			self.width = self.json.get('width', None)
			self.height = self.json.get('height', None)
			self.url = self.json.get('url', None)
			self.thumbnail = self.json.get('thumbnail', None)



class User:
	def __init__(self, data: dict = {}):
		self.json = data
		account = self.json.get('account', {})
		userProfile = self.json.get('userProfile', {})

		self.sid = self.json.get('sId', None)
		self.secret = self.json.get('secret', None)
		self.uid = account.get('uid', None)
		self.email = account.get('email', None)
		self.createdTime = account.get('createdTime', None)
		self.deviceId = account.get('deviceId', None)
		self.googleId = account.get('googleId', None)
		self.nickname = userProfile.get('nickname', None)
		self.socialId = userProfile.get('socialId', None)
		self.gender = userProfile.get('gender', None)
		self.socialIdModified = userProfile.get('socialIdModified', None)



class FromLink:
	def __init__(self, data: dict = {}):
		self.json = data
		self.path = self.json.get('path', None)
		self.objectId = self.json.get('objectId', None)
		self.objectType = self.json.get('objectType', None)
		self.parentId = self.json.get('parentId', None)
		self.parentType = self.json.get('parentType', None)
		self.shareLink = self.json.get('shareLink', None)

class ThreadList:
	def __init__(self, data: dict = {}):
		self.json = data
		self.threadId = list()
		self.title = list()
		self.hostUid = list()

		for thread in self.json:
			self.threadId.append(thread.get('threadId', None))
			self.hostUid.append(thread.get('hostUid', None))
			self.title.append(thread.get('title', None))


class Thread:
	def __init__(self, data: dict = {}):
		self.json = data
		self.beMentionedThreadIdList = self.json.get('beMentionedThreadIdList', None)
		self.isEnd = self.json.get('isEnd', None)
		self.ThreadList = ThreadList(self.json.get('list', []))
		self.threadCheckList = self.json.get('threadCheckList', None)


class Event:
	def __init__(self, data: dict = {}):
		self.json = data
		msg = self.json.get('msg', {})
		author = msg.get('author', {})
		extensions = msg.get('extensions', {})
		richFormat = msg.get('richFormat', {})
		authorExtensions = author.get('extensions', {})

		self.type = self.json.get('t', None)
		self.threadId = msg.get('threadId', None)
		self.messageId = msg.get('messageId', None)
		self.userId = author.get('uid', None)
		self.createdTime = msg.get('createdTime', None)
		self.messageType = msg.get('type', None)
		self.asSummary = msg.get('asSummary', None)
		self.seqId = msg.get('seqId', None)
		self.content = msg.get('content', None)
		self.roleContentStatus = extensions.get('roleContentStatus', None)
		self.richFormatVersion = richFormat.get('version', None)
		self.nickname = author.get('nickname', None)
		self.gender = author.get('gender', None)
		self.status = author.get('status', None)
		self.icon = Media(author.get('icon', {}))
		self.adminPartyOnlineStatus = authorExtensions.get('adminPartyOnlineStatus', None)
		self.bubbleColor = msg.get('bubbleColor', None)
		self.ref = msg.get('ref', {})



class ActivateShop:
	def __init__(self, data: dict = {}):
		self.json=data

		background = self.json.get('background', {})
		extensions = self.json.get('extensions', {})

		self.storeId = self.json.get('storeId', None)
		self.storeType = self.json.get('storeType', None)
		self.contentRegion = self.json.get('contentRegion', None)
		self.storeStatus = self.json.get('storeStatus', None)
		self.status = self.json.get('status', None)
		self.name = self.json.get('name', None)
		self.description = self.json.get('description', None)
		self.icon = Media(self.json.get('icon', None))
		self.backgroundColor = background.get('backgroundColor', None)
		self.displayMode = background.get('displayMode', None)

		self.ownerId = self.json.get('ownerId', None)
		self.ownerType = self.json.get('ownerType', None)
		self.lastActivateTime = extensions.get('lastActivateTime', None)
		self.isShowDefaultIcon = extensions.get('isShowDefaultIcon', None)
		self.isShowDefaultDescription = extensions.get('isShowDefaultDescription', None)
		self.createdTime = self.json.get('createdTime', None)
		self.owner = self.Owner(self.json.get('owner', {}))


	class Owner:
		def __init__(self, data: dict = {}):
			self.json=data

			extensions = self.json.get('extensions', {})
			userVisitorInfo = self.json.get('userVisitorInfo', {})

			self.userId = self.json.get('uid', None)
			self.nickname = self.json.get('nickname', None)
			self.socialId = self.json.get('socialId', None)
			self.socialIdModified = self.json.get('socialIdModified', None)
			self.gender = self.json.get('gender', None)
			self.status = self.json.get('status', None)
			self.storeId = self.json.get('storeId', None)
			self.icon = Media(self.json.get('icon', {}))
			self.chatInvitationStatus = self.json.get('chatInvitationStatus', None)
			self.publicChatInvitationStatus = self.json.get('publicChatInvitationStatus', None)
			self.privateChatInvitationStatus = self.json.get('privateChatInvitationStatus', None)
			self.circleInvitationStatus = self.json.get('circleInvitationStatus', None)
			self.adminPartyOnlineStatus = extensions.get('adminPartyOnlineStatus', None)
			self.openDaysInRow = extensions.get('openDaysInRow', None)
			self.maxOpenDaysInRow = extensions.get('maxOpenDaysInRow', None)
			self.lastOpenDate = extensions.get('lastOpenDate', None)
			self.showRecentVisitor = extensions.get('showRecentVisitor', None)
			self.onlineStatus = self.json.get('onlineStatus', None)
			self.createdTime = self.json.get('createdTime', None)
			self.contentRegion = self.json.get('contentRegion', None)
			self.contentRegionName = self.json.get('contentRegionName', None)
			self.userMood = self.UserMood(self.json.get('userMood', {}))
			self.showsSchool = self.json.get('showsSchool', None)
			self.lastActiveTime = self.json.get('lastActiveTime', None)
			self.showsLocation = self.json.get('showsLocation', None)
			self.location = self.json.get('location', None)
			self.nameCardEnabled = self.json.get('nameCardEnabled', None)
			self.matchEnabled = self.json.get('matchEnabled', None)
			self.tagline = self.json.get('tagline', None)
			self.totalViewCount = userVisitorInfo.get('totalViewCount', None)
			self.unreadViewCount = userVisitorInfo.get('unreadViewCount', None)
			self.userProfileVisitMode = userVisitorInfo.get('userProfileVisitMode', None)


		class UserMood:
			def __init__(self, data: dict = {}):
				self.json = data
				self.type = self.json.get('type', None)
				self.stickerId = self.json.get('stickerId', None)
				self.text = self.json.get('text', None)
				self.onlineStatus = self.json.get('onlineStatus', None)
				self.sticker = self.Sticker(self.json.get('sticker', {}))

			class Sticker:
				def __init__(self, data: dict = {}):
					self.json = data
					self.stickerId = self.json.get('stickerId', None)
					self.name = self.json.get('name', None)
					self.media = Media(self.json.get('media', {}))



class WalletInfo:
	def __init__(self, data: dict = {}):
		self.json = data
		self.userId = self.json.get('uid', None)
		self.activateStatus = self.json.get('activateStatus', None)
		self.status = self.json.get('status', None)
		self.createdTime = self.json.get('createdTime', None)
		self.walletExtension = self.json.get('walletExtension', {})
		self.unreadTransferOrderCount = self.json.get('unreadTransferOrderCount', None)
		self.latestTransferOrder = self.LatestTransfer(self.json.get('latestTransferOrder', {}))
		self.regularAccount = self.Account(self.json.get('regularAccount', {}))
		self.storeAccount = self.Account(self.json.get('storeAccount', {}))

	class LatestTransfer:
		def __init__(self, data: dict = {}):
			self.json = data

			extensions = self.json.get('extensions', {})

			self.orderId = self.json.get('orderId', None)
			self.orderType = self.json.get('orderType', None)
			self.currencyType = self.json.get('currencyType', None)
			self.orderStatus = self.json.get('orderStatus', None)
			self.fromUid = self.json.get('fromUid', None)
			self.toUid = self.json.get('toUid', None)
			self.amount = self.json.get('amount', None)
			self.duration = self.json.get('duration', None)
			self.minClaimedTime = self.json.get('minClaimedTime', None)
			self.createdTime = self.json.get('createdTime', None)
			self.returnTime = self.json.get('returnTime', None)
			self.giftBoxId = extensions.get('status', None)
			self.expiredTime = self.json.get('expiredTime', None)
			self.currency = self.Currency(self.json.get('currency', {}))


		class Currency:
			def __init__(self, data: dict = {}):
				self.json = data
				self.name = self.json.get('name', None)
				self.symbol = self.json.get('symbol', None)
				self.icon = Media(self.json.get('icon', {}))
				self.decimals = self.json.get('decimals', None)
				self.currencyType = self.json.get('currencyType', None)
				self.amount = self.json.get('amount', None)





	class Account:
		def __init__(self, data: dict = {}):
			self.json = data
			self.accountId = self.json.get('accountId', None)
			self.accountType = self.json.get('accountType', None)
			self.status = self.json.get('status', None)
			self.createdTime = self.json.get('createdTime', None)
			self.ownerId = self.json.get('ownerId', None)
			self.ownerType = self.json.get('ownerType', None)
			self.currencyList = self.json.get('currencyList', None)




class ChatInfo:
	def __init__(self, data: dict = {}):
		self.json = data



class CircleInfo:
	def __init__(self, data: dict = {}):
		self.json = data


class Comments:
	def __init__(self, data: dict = {}):
		self.json = data



class MentionCandidates:
	def __init__(self, data: dict = {}):
		self.json = data


class Blogs:
	def __init__(self, data: dict = {}):
		self.json = data



class Baners:
	def __init__(self, data: dict = {}):
		self.json = data


class Nfts:
	def __init__(self, data: dict = {}):
		self.json = data


from setuptools import setup, find_packages

with open("README.md", "r") as file:
	long_description = file.read()

link = 'https://github.com/xXxCLOTIxXx/projectZ.py/archive/refs/heads/main.zip'
ver = '1.1.6'

setup(
	name = "projectZ.py",
	version = ver,
	url = "https://github.com/xXxCLOTIxXx/projectZ.py",
	download_url = link,
	license = "MIT",
	author = "Xsarz",
	author_email = "xsarzy@gmail.com",
	description = "Library for creating projectZ bots and scripts.",
	long_description = long_description,
	long_description_content_type = "text/markdown",
	keywords = [
		"projectZ.py",
		"projectZ",
		"projectZ-py",
		"projectZ-bot",
		"api",
		"python",
		"python3",
		"python3.x",
		"xsarz",
		"official"
	],
	install_requires = [
		"colored",
		"requests",
		"websocket-client",
	],
	packages = find_packages()
)




from .utils import exceptions, generator, headers, objects
from .socket import Socket, CallBacks

from json import dumps, loads
from requests import Session
from random import randint
from sys import maxsize
from uuid import UUID
from typing import BinaryIO
from binascii import hexlify
from os import urandom
from threading import Thread
gen = generator.Generator()

class Client(Socket, CallBacks):
	def __init__(self, deviceId: str = None, proxies: dict = None, socket_debug: bool = False, sock_trace: bool = False, language: str = "en-US", country_code: str = "en", time_zone: int = 180):
		self.api = 'https://api.projz.com'
		self.session = Session()
		self.proxies = None
		self.deviceId = deviceId if deviceId else gen.deviceId()
		self.profile = objects.User()
		self.language = language
		self.country_code = country_code
		self.time_zone = time_zone

		Socket.__init__(self, self, sock_trace=sock_trace, debug=socket_debug)
		CallBacks.__init__(self)


	def parse_headers(self, endpoint: str, data = None, content_type: str = 'application/json') -> dict:
		h = headers.Headers(deviceId=self.deviceId, sid=self.profile.sid, time_zone=self.time_zone, country_code=self.country_code, language=self.language)
		head = h.get_persistent_headers()
		head.update(h.Headers())
		head.update({"Content-Type": content_type} if content_type else {})
		head["HJTRFS"] = gen.signature(path=endpoint, headers=head, body=data or bytes())
		return head

	def set_proxies(self, proxy = None):
		if type(proxy) == dict:
			self.proxies = proxy
		elif type(proxy) == str:
			self.proxies={"http": proxy, "https": proxy}
		elif proxy == None:
			self.proxies=None
		else:
			raise exceptions.WrongType(type(proxy))

	def upload_media(self, file: BinaryIO, target: int = 1):
		#TODO
		return None



	def login(self, email: str, password: str):

		data = dumps({
			"authType": 1,
			"email": email,
			"password": password
		})
		endpoint = '/v1/auth/login'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data, proxies=self.proxies)
		if response.status_code != 200:return exceptions.CheckException(response.text)
		else:
			self.profile = objects.User(loads(response.text))
			self.connect()
			return self.profile

	def logout(self):

		endpoint = '/v1/auth/logout'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		self.profile = objects.User()
		self.disconnect()
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code

	def Online(self):
		if self.online_loop_active: return
		
		self.online_loop_active = True
		Thread(target=self.online_loop).start()
		return self.online_loop_active

	def Offline(self):
		self.online_loop_active = False
		return self.online_loop_active

	def join_chat(self, chatId: int) -> None:

		endpoint = f'/v1/chat/threads/{chatId}/members'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code

	def leave_chat(self, chatId: int) -> None:

		endpoint = f'/v1/chat/threads/{chatId}/members'
		response = self.session.delete(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code

	def get_from_link(self, link: str):
		data = dumps({"link": link})

		endpoint = f"/v1/links/path"
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data, proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else objects.FromLink(loads(response.text))

	def get_link(self, userId: int):

		data = dumps({
			"objectId": 0,
			"objectType": 0,
			"parentId": 0,
			"path": f"user/{userId}",
			"circleIdForCircleAnnouncement": 0,
			"parentType": 0
		})

		endpoint = '/v1/links/share'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data, proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else objects.FromLink(loads(response.text))

	def get_my_chats(self, start: int = 0, size: int = 20, type: str = 'managed'):

		endpoint = f'/v1/chat/joined-threads?start={start}&size={size}&type={type}'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else objects.Thread(loads(response.text))


	def send_message(self, chatId: int, message: str, message_type: int = 1, reply_to: int = None, poll_id: int = None, dice_id: int = None):

		data = {
			"type": message_type,
			"threadId": chatId,
			"uid": self.profile.uid,
			"seqId": randint(0, maxsize),
			"extensions": {}
		}
		data["content"] = message

		if reply_to: data["extensions"]["replyMessage"] = reply_to
		if poll_id: data["extensions"]["pollId"] = poll_id
		if dice_id: data["extensions"]["diceId"] = dice_id

		resp = self.send(t=1, data=data, threadId=chatId)
		return resp

	def get_verify_code(self, email: str):

		data = dumps({
			"authType": 1,
			"purpose": 1,
			"email": email,
			"password": "",
			"phoneNumber": "",
			"securityCode": "",
			"invitationCode": "",
			"secret": "",
			"gender": 0,
			"birthday": "1990-01-01",
			"requestToBeReactivated": False,
			"countryCode": self.country_code,
			"suggestedCountryCode": self.country_code.upper(),
			"ignoresDisabled": True,
			"rawDeviceIdThree": gen.generate_device_id_three()
		})

		endpoint = '/v1/auth/request-security-validation'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data, proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code

	def register(self, email: str, password: str, code: str, icon: BinaryIO, invitation_code: str = None, nickname: str = 'XsarzyBest', tag_line: str = 'XsarzBot', gender: int = 100, birthday: str = '1990-01-01'):

		data = dumps({
			"authType": 1,
			"purpose": 1,
			"email": email,
			"password": password,
			"securityCode": code,
			"invitationCode": invitation_code or "",
			"nickname": nickname,
			"tagLine": tag_line,
			"icon": self.upload_media(target=1, file=icon),
			"nameCardBackground": None,
			"gender": gender,
			"birthday": birthday,
			"requestToBeReactivated": False,
			"countryCode": self.country_code,
			"suggestedCountryCode": self.country_code.upper(),
			"ignoresDisabled": True,
			"rawDeviceIdThree": gen.generate_device_id_three()
		})

		endpoint = '/v1/auth/register'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data, proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code


	def visit(self, userId):

		endpoint = f'/v1/users/profile/{userId}/visit'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code

	def activate_wallet(self, wallet_password: str, code: str, email: str = None):


		data = dumps({
			"authType": 1,
			"identity": email if email else self.profile.email if self.profile.email else exceptions.NotLoggined('You are not authorized'),
			"paymentPassword": wallet_password,
			"securityCode": code
		})

		endpoint = '/biz/v1/wallet/0/activate'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data, proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code

	def activate_shop(self):

		endpoint = '/biz/v1/activate-store'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else objects.ActivateShop(loads(response.text))

	def wallet_info(self):

		endpoint = '/biz/v1/wallet'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else  objects.WalletInfo(loads(response.text))







	def my_nfts(self):

		endpoint = '/biz/v1/nfts/count'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else objects.Nfts(loads(response.text))


	def get_baners(self):

		endpoint = '/v2/banners'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else objects.Baners(loads(response.text))

	def get_circles(self, type: str = 'recommend', categoryId: int = 0, size: int = 10):

		endpoint = f'/v1/circles?type={type}&categoryId={categoryId}&size={size}'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.text

	def get_blocked_users(self):

		endpoint = '/v1/users/block-uids'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.text

	def get_blogs(self, type: str = 'recommend', size: int = 10):

		endpoint = f'/v1/blogs?type={type}&size={size}'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else objects.Blogs(loads(response.text))


	def mark_as_read(self, chatId: int):

		endpoint = f'/v1/chat/threads/{chatId}/mark-as-read'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code

	def get_chat_threads(self, chatId: int):

		endpoint=f'/v1/chat/threads/{chatId}'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.text

	def get_online_chat_members(self, chatId: int):

		endpoint=f'/v1/chat/threads/{chatId}/online-members'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.text


	def get_chat_messages(self, chatId: int, size: int = 10):

		endpoint=f'/v1/chat/threads/{chatId}/messages?size={size}'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.text

	def get_mention_candidates(self, chatId: int, size: int = 10, queryWord: str = ''):
		
		endpoint = f'/v1/chat/threads/{chatId}/mention-candidates?size={size}&queryWord={queryWord}'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else  objects.MentionCandidates(loads(response.text))

	def comment(self, message: str, userId: int = None, blogId: int = None, replyId: dict = None):

		data = {
			"commentId": 0,
			"status":1,
			"parentId": userId,
			"replyId": 0,
			"circleId": 0,
			"uid": 0,
			"content": message,
			"mediaList": [],
			"commentType": 1,
			"subComments": [],
			"subCommentsCount": 0,
			"isPinned": False
		}

		if userId:
			data['parentType'] = 4

		elif blogId:
			data['parentType'] = 2

		else:
			raise exceptions.WrongType()


		if replyId:
			data['replyId'] = replyId['commentId']
			data['extensions'] = {"replyToUid": replyId['userId'], "contentStatus": 1}

		data = dumps(data)
		endpoint = f'/v1/comments'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data, proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else objects.Comments(loads(response.text))


	def get_comments(self, userId: int, type: int = 4, replyId: int= 0, size: int = 30, onlyPinned: int = 0):

		endpoint = f'/v1/comments?parentId={userId}&parentType={type}&replyId={replyId}&size={size}&onlyPinned={onlyPinned}'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else objects.Comments(loads(response.text))


	def block(self, userId: int):

		endpoint = f'/v1/users/block/{userId}'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code

	def unblock(self, userId: int):

		endpoint = f'/v1/users/block/{userId}'
		response = self.session.delete(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code



	def accept_chat_invitation(self, chatId: int):

		endpoint = f'/v1/chat/threads/{chatId}/accept-invitation'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code


	def join_circle(self, circleId):
		data = dumps({"joinMethod": 1})

		endpoint = f'/v1/circles/{circleId}/members'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data, proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code


	def leave_circle(self, circleId):

		endpoint = f'/v1/circles/{circleId}/members'
		response = self.session.delete(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code


	def get_circle_info(self, circleId: int):

		endpoint = f'/v1/circles/{circleId}'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else objects.CircleInfo(loads(response.text))


	def get_chat_info(self, chatId: int):

		endpoint = f'/v1/chat/threads/{chatId}'
		response = self.session.get(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else objects.ChatInfo(loads(response.text))

	def follow(self, userId: int):

		endpoint = f'/v1/users/membership/{userId}'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code

	def unfollow(self, userId: int):

		endpoint = f'/v1/users/membership/{userId}'
		response = self.session.delete(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code



	def like(self, commentId: int = None, blogId: int = None, stickerId: int = 65956773102028339):

		data = {
			"createdTime": 0,
			"stickerId": stickerId,
			"count": 0,
			"justAddTimeMs": 0
		}

		if commentId:
			data['objectType'] = 3
			data['objectId'] = commentId

		elif blogId:
			data['objectType'] = 2
			data['objectId'] = blogId

		else:
			raise exceptions.WrongType()

		data = dumps(data)
		endpoint = f'/v1/reactions'
		response = self.session.post(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint, data=data), data=data, proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code


	def unlike(self, commentId: int = None, blogId: int = None, stickerId: int = 65956773102028339):

		if commentId:
			endpoint = f'/v1/reactions?objectId={commentId}&objectType=3&stickerId={stickerId}'
		elif blogId:
			endpoint = f'/v1/reactions?objectId={blogId}&objectType=2&stickerId={stickerId}'
		else:
			raise exceptions.WrongType()

		response = self.session.delete(f"{self.api}{endpoint}", headers=self.parse_headers(endpoint=endpoint), proxies=self.proxies)
		return exceptions.CheckException(response.text) if response.status_code != 200 else response.status_code


from websocket import WebSocketApp, enableTrace
from websocket import _exceptions as WSexceptions
from .utils import exceptions
from threading import Thread
from time import sleep
from json import dumps, loads
from .utils import objects

class Socket:
	def __init__(self, client, debug: bool = False, sock_trace: bool = False):
		self.socket_url = "wss://ws.projz.com"
		self.socket = None
		self.debug=debug

		self.client = client
		self.active = False
		self.online_loop_active = False
		self.socket_thread = None
		self.reconnectTime = 90
		self.reconnect_thread = None
		self.buffer = list()
		enableTrace(sock_trace)


	def resolve(self, ws, data):
		data = loads(data)
		try:
			messageId = data['msg']['messageId']
			if messageId in self.buffer:
				return
			self.buffer.append(messageId)
			#I know, it looks like shit, but I haven't come up with another way yet (when the socket is rebooted, all messages that have already come come again)
		except:
			pass
		self.on_all(data)
		self.methods.get(data['t'])(data)

	def connect(self):
		try:
			if self.debug:
				print(f"[socket][start] Starting Socket")

			self.socket = WebSocketApp(
				f"{self.socket_url}/v1/chat/ws",
				header = self.client.parse_headers(endpoint='/v1/chat/ws'),
				on_message=self.resolve
			)
			self.socket_thread = Thread(target=self.socket.run_forever)
			self.socket_thread.start()
			self.active = True
			if self.reconnect_thread is None:
				self.reconnect_thread = Thread(target=self.reconnect)
				self.reconnect_thread.start()
			if self.debug:
				print(f"[socket][start] Socket Started")
		except Exception as e:
			if self.debug:
				print(f"[socket][start] Error while starting Socket : {e}")

	def disconnect(self):
		if self.debug:
			print(f"[socket][close] Closing Socket")
		try:
			self.socket.close()
			self.active = False
			self.socket_thread = None
		except Exception as e:
			if self.debug:
				print(f"[socket][close] Error while closing Socket : {e}")
		return

	def reconnect(self):
		while True:
			sleep(self.reconnectTime)
			if self.active:
				if self.debug is True:
					print(f"[socket][reconnect_handler] Reconnecting Socket")
				self.disconnect()
				self.connect()

	def online_loop(self):
		sleep(1.5)
		while self.online_loop_active:
			try:
				self.send()
			except WSexceptions.WebSocketConnectionClosedException:
				sleep(1.5)
				continue
			except Exception as e:
				if self.debug:
					print('[socket][_online_loop][error] ', e)
			sleep(3)



	def send(self, t: int = 8, data = None, threadId: int = None):
		
		if not self.socket_thread:
			raise exceptions.NotLoggined('You are not logged in')
		d = {'t': t}
		if threadId:d['threadId'] = threadId
		if data:d['msg'] = data
		if self.debug is True:
			print(f"[socket][send] Sending Data : {d}")
		self.socket.send(dumps(d))


class CallBacks:
	def __init__(self):

		self.handlers = {}

		self.methods = {
			1: self.on_text_message,
			2: self.on_image_message,
			3: self.on_audio_message,
			4: self.on_video_message,
			5: self.on_delete_message,
			10: self.on_user_join,
			11: self.on_user_leave,
			12: self.on_user_invite,
			13: self.on_user_kick,
			14: self.on_user_remove,
			15: self.on_cohost_remove,
			16: self.on_cohost_add,
			17: self.on_host_delete_message,
			18: self.on_cohost_delete_message,
			20: self.on_role_play_invite,
			21: self.on_free_talk_start,
			22: self.on_free_talk_end,
			23: self.on_role_play_start,
			24: self.on_role_play_end,
			25: self.on_voice_call_start,
			26: self.on_voice_call_end,
			27: self.on_voice_call_reject,
			28: self.on_voice_call_cancel,
			29: self.on_voice_call_accept,
			30: self.on_free_talk_add_user,
			31: self.on_free_talk_remove_user,
			32: self.on_free_talk_invite,
			33: self.on_free_talk_apply,
			34: self.on_free_talk_accept,
			35: self.on_live_talking_users,
			36: self.on_free_talk_apply_count,
			37: self.on_role_play_roles,
			38: self.on_role_play_role_update,
			39: self.on_role_play_apply_count,
			40: self.on_conversation_level,
			41: self.on_role_play_accept_apply,
			42: self.on_role_play_user_role_remove,
			43: self.on_chat_user_online,
			50: self.on_chat_delete,
			51: self.on_chat_host_update,
			53: self.on_chat_disable,
			60: self.on_user_typing,
			90: self.on_chat_activity_type,
			120: self.on_voice_call_not_answered,
		}


	def event(self, type: str):
		def registerHandler(handler):
			if type in self.handlers:
				self.handlers[type].append(handler)
			else:
				self.handlers[type] = [handler]
			return handler
		return registerHandler

	def call(self, type, data):
		if type in self.handlers:
			for handler in self.handlers[type]:
				handler(objects.Event(data))


	def on_text_message(self, data): self.call(type='on_text_message', data=data)
	def on_image_message(self, data): self.call(type='on_image_message', data=data)
	def on_audio_message(self, data): self.call(type='on_audio_message', data=data)
	def on_video_message(self, data): self.call(type='on_video_message', data=data)
	def on_delete_message(self, data): self.call(type='on_delete_message', data=data)
	def on_user_join(self, data): self.call(type='on_user_join', data=data)
	def on_user_leave(self, data): self.call(type='on_user_leave', data=data)
	def on_user_invite(self, data): self.call(type='on_user_invite', data=data)
	def on_user_kick(self, data): self.call(type='on_user_kick', data=data)
	def on_user_remove(self, data): self.call(type='on_user_remove', data=data)
	def on_cohost_remove(self, data): self.call(type='on_cohost_remove', data=data)
	def on_cohost_add(self, data): self.call(type='on_cohost_add', data=data)
	def on_host_delete_message(self, data): self.call(type='on_host_delete_message', data=data)
	def on_cohost_delete_message(self, data): self.call(type='on_cohost_delete_message', data=data)
	def on_role_play_invite(self, data): self.call(type='on_role_play_invite', data=data)
	def on_free_talk_start(self, data): self.call(type='on_free_talk_start', data=data)
	def on_free_talk_end(self, data): self.call(type='on_free_talk_end', data=data)
	def on_role_play_start(self, data): self.call(type='on_role_play_start', data=data)
	def on_role_play_end(self, data): self.call(type='on_role_play_end', data=data)
	def on_voice_call_start(self, data): self.call(type='on_voice_call_start', data=data)
	def on_voice_call_end(self, data): self.call(type='on_voice_call_end', data=data)
	def on_voice_call_reject(self, data): self.call(type='on_voice_call_reject', data=data)
	def on_voice_call_cancel(self, data): self.call(type='on_voice_call_cancel', data=data)
	def on_voice_call_accept(self, data): self.call(type='on_voice_call_accept', data=data)
	def on_free_talk_add_user(self, data): self.call(type='on_free_talk_add_user', data=data)
	def on_free_talk_remove_user(self, data): self.call(type='on_free_talk_remove_user', data=data)
	def on_free_talk_invite(self, data): self.call(type='on_free_talk_invite', data=data)
	def on_free_talk_apply(self, data): self.call(type='on_free_talk_apply', data=data)
	def on_free_talk_accept(self, data): self.call(type='on_free_talk_accept', data=data)
	def on_live_talking_users(self, data): self.call(type='on_live_talking_users', data=data)
	def on_free_talk_apply_count(self, data): self.call(type='on_free_talk_apply_count', data=data)
	def on_role_play_roles(self, data): self.call(type='on_role_play_roles', data=data)
	def on_role_play_role_update(self, data): self.call(type='on_role_play_role_update', data=data)
	def on_role_play_apply_count(self, data): self.call(type='on_role_play_apply_count', data=data)
	def on_conversation_level(self, data): self.call(type='on_conversation_level', data=data)
	def on_role_play_accept_apply(self, data): self.call(type='on_role_play_accept_apply', data=data)
	def on_role_play_user_role_remove(self, data): self.call(type='on_role_play_user_role_remove', data=data)
	def on_chat_user_online(self, data): self.call(type='on_chat_user_online', data=data)
	def on_chat_delete(self, data): self.call(type='on_chat_delete', data=data)
	def on_chat_host_update(self, data): self.call(type='on_chat_host_update', data=data)
	def on_chat_disable(self, data): self.call(type='on_chat_disable', data=data)
	def on_user_typing(self, data): self.call(type='on_user_typing', data=data)
	def on_chat_activity_type(self, data): self.call(type='on_chat_activity_type', data=data)
	def on_voice_call_not_answered(self, data): self.call(type='on_voice_call_not_answered', data=data)

	def on_all(self, data): self.call(type='on_all', data=data)